@@ -16,9 +16,11 @@ For more examples, see the tests/ directory in the Instant distribution.
 Questions, bugs and patches should be sent to fenics-dev@googlegroups.com.
 """
 
+import pkg_resources
+
 __authors__ = "Magne Westlie, Kent-Andre Mardal <kent-and@simula.no>, Martin Alnes <martinal@simula.no>, Ilmar M. Wilbers <ilmarw@simula.no>"
 __date__ = "2016-11-30"
-__version__ = "2016.2.0"
+__version__ = pkg_resources.get_distribution("fenics-instant").version
 
 # TODO: Import only the official interface
 from .output import *
