@@ -21,7 +21,7 @@
 # Alternatively, Instant may be distributed under the terms of the BSD license.
 
 from six import string_types
-import logging, os, platform, sys
+import io, logging, os, platform, sys
 
 # Logging wrappers
 _log = logging.getLogger("instant")
@@ -106,12 +106,14 @@ def instant_assert(condition, *message):
 # Utility functions for file handling:
 
 
-def write_file(filename, text):
+def write_file(filename, text, mode="w"):
     "Write text to a file and close it."
     try:
-        f = open(filename, "w")
-        f.write(text)
-        f.close()
+        if isinstance(text, bytes):
+            text = text.decode("utf8")
+        with io.open(filename, mode, encoding="utf8") as f:
+            f.write(text)
+            f.flush()
     except IOError as e:
         instant_error("Can't open '%s': %s" % (filename, e))
 
